*====================================================================
* VSSProject.Prg
*
* Updates the information in the Visual FoxPro project. It automati-
* cally updates the working folder to be the same as the project's
* directory. It also changes the VSS project to which the project is
* attached. You must run this tool in any of the following circum-
* stances:
*
* - You want to attach the PJX file to a different VSS database, for
*   example, because you work on a different size without being able
*   to check in to the master database regularily.
*
* - You copied the PJX file into a different directory.
*
* - You changed the project tree in VSS.
*
* - You got a copy of the current project from a fellow and want to
*   use the new PJX files instead of letting VFP re-create it. This
*   is an important issue, if your projects consists of dozens of
*   PJX files.
*
*--------------------------------------------------------------------
* SYNTAX
*
*  VSSProject( "<PJX-Project>", "<VSS-Project>", @cMessage )
*
*  If you specify a cMessage parameter, the message is returned in
*  case of an error. Otherwise, it's displayed in a messagebox. You
*  can use this option for batch processing. cMessage must be 
*  initialized as a string
*
*  This program returns an error number, if something went wrong. 
*  Please see the code for specific numbers.
*
*====================================================================
LParameter tcPJXProject, tcVSSProject, rcMsg

	*-----------------------------------------------------------------
	* Verify the VFP Project parameter
	*-----------------------------------------------------------------
	Local lcPJX
	If VarType(m.tcPJXProject) # "C"
		Return SCCError("PJX file must be specified.",@rcMsg,1)
	Endif
	lcPJX = ForceExt( Alltrim(m.tcPJXProject), "PJX" )
	If not File(m.lcPJX)
		Return SCCError( ;
			"Project '"+m.lcPJX+"' does not exist.", @rcMsg, 2 ;
		)
	Endif
	
	*-----------------------------------------------------------------
	* Verify the VSS project parameter
	*-----------------------------------------------------------------
	Local lcVSS
	If VarType(m.tcVSSProject) # "C"
		Return SCCError("VSS project must be specified.",@rcMsg,3)
	Endif
	lcVSS = Upper(Alltrim(m.tcVSSProject))
	If not Left(m.lcVSS,2) == "$/"
		lcVSS = "$/" + m.lcVSS
	Endif
	
	*-----------------------------------------------------------------
	* Open the Visual FoxPro project file. If the project is not
	* added to SourceSafe, we display a message.
	*-----------------------------------------------------------------
	Local lnSelect, lnOK, lcError, lnError, lcSCCDATA
	lnSelect = Select()
	lnOK = 0
	Select 0
	lnError = 0
	lcError = On( "Error" )
	On Error lnError = Error()
	Use (m.lcPJX) Again Shared
	On Error &lcError
	If m.lnError > 0
		lnOK = 4
		SCCError( "Cannot open project file", @rcMsg )
	Endif
	If m.lnOK == 0
		lcSCCDATA = SCCDATA
		If Empty(m.lcSCCData)
			lnOK = 5
			SCCError( "Project not under version control.", @rcMsg )
		Endif
		If Len(m.lcSCCData) # 1048
			lnOK = 6
			SCCError( "VSS info record corrupt.", @rcMsg )
		Endif
	Endif
	
	*-----------------------------------------------------------------
	* Get the name of the VSS project. We only replace it when it 
	* differs, because this string contains one piece of information
	* that we can't find out: The filename of the PJM file within the
	* VSS database. VFP can work just fine without this particular
	* peice, but why remove it if it's not necessary?
	*-----------------------------------------------------------------
	Local lcVSSProject, lcVSSProjectName, lnLength, lcVSSPJM
	If m.lnOK == 0
		lcVSSProject = SCCExtract( m.lcSCCDATA, 9, 260 )
		lnLength = At(",",m.lcVSSProject) - 3
		If m.lnLength <= 0
			lcVSSProjectName = ""
			lcVSSPJM = ""
		Else
			lcVSSProjectName = SubStr(m.lcVSSProject,2,m.lnLength)
			lcVSSPJM = Right( ;
				m.lcVSSProject, ;
				Len(m.lcVSSProject) - m.lnLength - 3 ;
			)
		Endif
		If not Upper(m.lcVSSProjectName) == m.lcVSS
			lcSCCDATA = Stuff( ;
				m.lcSCCDATA, ;
				9, ;
				260, ;
				Padr( ["]+m.lcVSS+[",], 260, Chr(0) ) ;
			)
		Endif
	Endif
	
	*-----------------------------------------------------------------
	* Change the name of the project file and the working folder.
	*-----------------------------------------------------------------
	If m.lnOK == 0
		lcSCCDATA = Stuff( ;
			m.lcSCCDATA, ;
			529, ;
			260, ;
			Padr(Dbf(),260,Chr(0)) ;
		)
		lcSCCDATA = Stuff( ;
			m.lcSCCDATA, ;
			789, ;
			260, ;
			Padr(JustPath(Dbf()),260,Chr(0)) ;
		)
	Endif
	
	*-----------------------------------------------------------------
	* Write back the new SCCDATA record and clean up.
	*-----------------------------------------------------------------
	If m.lnOK == 0
		Replace next 1 SCCData with m.lcSCCDATA
	Endif
	Use
	Select (m.lnSelect)
	
Return m.lnOK


*====================================================================
* Extracts a string out of the binary structure in SCCDATA.
*====================================================================
Procedure SCCExtract
LParameter tcSCCData, tnPos, tnLength

	*-----------------------------------------------------------------
	* Get the string out of the SCCDATA field
	*-----------------------------------------------------------------
	Local lcString
	If Len(m.tcSCCData) < m.tnPos+m.tnLength-1
		lcString = ""
	Else
		lcString = SubStr( m.tcSCCData, m.tnPos, m.tnLength )
	Endif
	
	*-----------------------------------------------------------------
	* Strings in SCCDATA are filled with CHR(0). Because the first 
	* CHR(0) character terminates the string, there might potentially
	* be non-sense characters after that. I've not yet seen this, but
	* I'm rather careful.
	*-----------------------------------------------------------------
	Local lnPos
	lnPos = At( Chr(0), m.lcString )
	If m.lnPos > 0
		lcString = Left( m.lcString, m.lnPos-1 )
	Endif
	
Return m.lcString


*====================================================================
* Either displays a message or stores the message in the variable
* that is passed by reference.
*====================================================================
Procedure SCCError
LParameter tcError, tnCode, rcMessage

	If VarType(m.rcMessage) == "C"
		rcMessage = m.tcError
	Else
		MessageBox( m.tcError, 16, "VSSProj.Prg" )
	Endif
	
Return m.tnCode
